/*
 * Copyright (C) 2021 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.zipflinger;

import java.io.IOException;
import java.io.InputStream;
import java.util.Random;

public class MockInputStream extends InputStream {

    private long size;
    private final Random random;

    MockInputStream(long size) {
        this.size = size;
        this.random = new Random(0);
    }

    @Override
    public int read() throws IOException {
        if (size == 0) {
            return -1;
        }
        size -= 1;
        return random.nextInt(256);
    }
}
